#!/usr/bin/env node

let path = require('path');
let os = require('os');
let { spawnSync } = require('child_process');
let { getOptions } = require('../lib/utils/index.js');

let { log } = require('../lib/utils');
//initPreCommitHook();

let options = getOptions();

let { esLint: esLintOptions } = options || {};
let {
  ignoreFilePaths: esLintIgnorePaths,
  fix: esLintFix,
  isCI: isCI,
  reportPath: reportPath
} = esLintOptions || {};

let isWindows = os.platform().toLowerCase() === 'win32';

let [, , option] = process.argv;
let args = process.argv.slice(3);
let appPath = process.cwd();

const isNodeModuleUnderAppFolder = __dirname.indexOf(appPath) !== -1;

const _getCliPath = !isNodeModuleUnderAppFolder
  ? libName => path.join(__dirname, '..', 'node_modules', '.bin', libName)
  : libName => libName;
const suffixExt = isWindows ?  '.cmd' : '';

function getCliPath(libName) {
  return _getCliPath(libName) + suffixExt;
}

let webpack = getCliPath('webpack');
let nodemon = getCliPath('nodemon');
let babel = getCliPath('babel');
let propertyToJson = getCliPath('propertyToJson');
let esLint = getCliPath('eslint');

let result;
switch (option) {
  case 'lint-setup': {
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/lint/index.js')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;
  }
  case 'add-lint-scripts': {
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/lint/lintScripts.js')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;
  }
  case 'postpublish':
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/dependencyPostPublish')].concat(args),
      { stdio: 'inherit' }
    );

    process.exit(result.status);
    break;
  case 'rtl':
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/rtl')].concat(args),
      { stdio: 'inherit' }
    );

    process.exit(result.status);
    break;
  case 'app':
    result = spawnSync(
      'cp',
      ['-r', path.join(__dirname, '..', 'templates', 'app')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'library':
    result = spawnSync(
      'cp',
      ['-r', path.join(__dirname, '..', 'templates', 'library')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'move':
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/copy')].concat(args.concat(false)),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'copy':
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/copy')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'copyTimezones':
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/copyTimezones')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'clone':
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/repoClone')].concat(args.concat(args)),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'propertyToJson':
    result = spawnSync(
      propertyToJson,
      args.map(arg => path.join(appPath, arg)),
      {
        stdio: 'inherit'
      }
    );
    process.exit(result.status);
    break;

  case 'clean':
    args = args.map(arg => path.join(appPath, arg));
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/clean')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'start':
    result = spawnSync(
      'node',
      [require.resolve('../lib/servers/server'), '--expose-http2'].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;
  case 'mockserver':
    result = spawnSync(
      nodemon,
      [
        require.resolve('../lib/servers/mockserver.js'),
        '--delay',
        '2500ms',
        'watch',
        `${appPath + path.sep}mockapi`
      ].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'docs':
    result = spawnSync(
      'node',
      [require.resolve('../lib/servers/docsServer')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'build':
    result = spawnSync(
      webpack,
      [
        '--config',
        require.resolve('../lib/configs/webpack.prod.config.js')
      ].concat(args),
      { stdio: 'inherit' }
    );

    process.exit(result.status);
    break;

  case 'build:library:umd':
  case 'build:component:umd':
    result = spawnSync(
      webpack,
      [
        '--config',
        require.resolve('../lib/configs/webpack.component.umd.config.js')
      ],
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;
  case 'build:css:umd':
    result = spawnSync(
      webpack,
      ['--config', require.resolve('../lib/configs/webpack.css.umd.config.js')],
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;
  case 'build:component:server':
    result = spawnSync(
      webpack,
      ['--config', require.resolve('../lib/configs/webpack.server.config.js')],
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'build:library:cmjs':
  case 'build:component:cmjs':
    result = spawnSync(
      babel,
      [
        'src',
        '-d',
        'lib',
        `--presets=${require.resolve('../lib/babel/cmjs-plugins-presets.js')}`,
        '--copy-files'
      ].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'build:library:es':
  case 'build:component:es':
    result = spawnSync(
      babel,
      [
        'src',
        '--out-dir',
        'es',
        `--presets=${require.resolve('../lib/babel/es-plugins-presets.js')}`,
        '--copy-files'
      ].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'test':
    result = spawnSync(
      'node',
      [require.resolve('../lib/jest/run.js')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'sstest':
    result = spawnSync(
      'node',
      [require.resolve('../lib/servers/scrServer')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'publish:report':
    result = spawnSync(
      'sh',
      [path.join(__dirname, '..', 'lib', 'sh', 'reportPublish.sh')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'impact':
    result = spawnSync(
      'node',
      [require.resolve('../lib/server/impactServer')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'cluster:monitor':
    result = spawnSync(
      'node',
      [require.resolve('../lib/servers/clusterHubServer')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'lint':
    result = spawnSync(
      esLint,
      [
        '-c',
        path.join(__dirname, '..', '.eslintrc.js'),
        isCI ? '--no-eslintrc' : false,
        '--ignore-path',
        esLintIgnorePaths
          ? path.join(appPath, esLintIgnorePaths)
          : path.join(__dirname, '..', '.eslintignore'),
        esLintFix ? '--fix' : false,
        isCI ? '-f' : false,
        isCI ? path.join(__dirname, './../lib/utils/lintReporter.js') : false
      ]
        .filter(Boolean)
        .concat(args.map(arg => path.join(appPath, arg))),
      {
        stdio: 'inherit'
      }
    );
    isCI ? log(`You can see the eslint result from ${reportPath}`) : false;
    break;

  case 'set:config':
    result = spawnSync(
      'node',
      [require.resolve('../lib/utils/setEnvVariables')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case 'help':
    result = spawnSync(
      'node',
      [require.resolve('../lib/servers/helpServer')].concat(args),
      { stdio: 'inherit' }
    );
    process.exit(result.status);
    break;

  case '--version':
  case '-version':
  case '--v':
  case '-v':
    log(`@zohodesk/react-cli v${require('../package.json').version}`);
    break;

  default:
    log(`react-cli > Unknown option "${option}"`);
    log('react-cli app <appName>');
    log('react-cli library <libraryName>');
    log('react-cli start');
    log('react-cli build');
    log('react-cli sstest');
    log('react-cli test');
    log('react-cli publish:report');
    log('react-cli build:library:es');
    log('react-cli gitclone <last argument clone folder>');
    log('react-cli hgclone <last argument clone folder>');

    process.exit(0);
    break;
}
